import os, sys

import maya.cmds as cmds
import maya.OpenMayaMPx as OpenMayaMPx

TRANSLATOR_NAME = 'Verge3D glTF'
FILE_EXT = 'gltf'
VENDOR = 'Soft8Soft'
VERSION = '4.8.0'

class V3DTranslator(OpenMayaMPx.MPxFileTranslator):
    def __init__(self):
        OpenMayaMPx.MPxFileTranslator.__init__(self)
    def haveWriteMethod(self):
        return True
    def haveReadMethod(self):
        return False
    def filter(self):
        return '*.{}'.format(FILE_EXT)
    def defaultExtension(self):
        return FILE_EXT

    def writer(self, fileObj, optStr, accessMode):

        import const
        import pluginUtils
        log = pluginUtils.log.getLogger('V3D-MY')

        if const.V3D_PLUGIN_NAME not in cmds.pluginInfo(query=True, listPlugins=True):
            log.error('Verge3D GLTF Exporter requires Verge3D plugin to be loaded.')
            return

        fullName = fileObj.fullName()
        try:
            if accessMode == OpenMayaMPx.MPxFileTranslator.kExportAccessMode:
                import interface
                interface.exportGLTFPath(fileObj.fullName())

            elif accessMode == OpenMayaMPx.MPxFileTranslator.kExportActiveAccessMode:
                log.error('Export selection is not implemented yet')
        except:
            log.error('Failed to write file information')
            raise

    def reader(self, fileObject, optionString, accessMode):
        raise NotImplementedError()

    def identifyFile(self, fileObj, buffer, size):
        basename, ext = os.path.splitext(fileObj.fullName())

        if ext in ['.gltf', '.glb']:
            return self.kIsMyFileType
        else:
            return self.kNotMyFileType

def translatorCreator():
    return OpenMayaMPx.asMPxPtr(V3DTranslator())


def initializePlugin(mobject):
    mplugin = OpenMayaMPx.MFnPlugin(mobject, VENDOR, VERSION, 'Any')
    mplugin.setName('verge3dGLTFExporter')
    mpluginPath = mplugin.loadPath()

    sys.path.append(os.path.normpath(os.path.join(mpluginPath, 'lib')))
    sys.path.append(os.path.normpath(os.path.join(mpluginPath, '..', 'python')))
    import pluginUtils
    log = pluginUtils.log.getLogger('V3D-MY')

    try:
        mplugin.registerFileTranslator(TRANSLATOR_NAME, None, translatorCreator)
    except:
        log.error('Failed to register translator: {}'.format(TRANSLATOR_NAME))
        raise


def uninitializePlugin(mobject):
    mplugin = OpenMayaMPx.MFnPlugin(mobject)

    import pluginUtils
    log = pluginUtils.log.getLogger('V3D-MY')

    try:
        mplugin.deregisterFileTranslator(TRANSLATOR_NAME)
    except:
        log.error('Failed to unregister translator: {}'.format(TRANSLATOR_NAME))
        raise
